@@ -68,7 +68,7 @@ class CloudFlare():
                 del self.network
                 self.network = None
 
-        def _add_headers(self, method, content_type, data, files):
+        def _add_headers(self, method, data, files, content_type=None):
             """ Add default headers """
             self.headers = {}
             self.headers['User-Agent'] = self.user_agent
@@ -76,16 +76,30 @@ class CloudFlare():
                 # no content type needed - except we throw in a default just for grin's
                 self.headers['Content-Type'] = 'application/json'
             elif content_type is not None and method in content_type:
-                    # this api endpoint and this method requires a specific content type.
-                    ct = content_type[method]
-                    if isinstance(ct, list):
-                        # How do we choose from more than one content type?
-                        for t in ct:
-                            # we have to match against the data type - arggg! how? XXX/TODO - use first for now
+                # this api endpoint and this method requires a specific content type.
+                ct = content_type[method]
+                if isinstance(ct, list):
+                    # How do we choose from more than one content type?
+                    found = False
+                    for t in ct:
+                        # we have to match against the data type - arggg!
+                        if 'application/octet-stream' == t and isinstance(data, (bytes,bytearray)):
                             self.headers['Content-Type'] = t
+                            found = True
                             break
-                    else:
-                        self.headers['Content-Type'] = ct
+                        if 'application/json' == t and isinstance(data, (list,dict)):
+                            self.headers['Content-Type'] = t
+                            found = True
+                            break
+                        if 'application/javascript' == t and isinstance(data, str):
+                            self.headers['Content-Type'] = t
+                            found = True
+                            break
+                    if not found:
+                        # punt - pick first - we can't do anything else!
+                        self.headers['Content-Type'] = ct[0]
+                else:
+                    self.headers['Content-Type'] = ct
             else:
                 # default choice
                 self.headers['Content-Type'] = 'application/json'
@@ -98,18 +112,30 @@ class CloudFlare():
                 if self.headers['Content-Type'] == 'application/json' and isinstance(data, (bytes,bytearray)):
                     # passing binary file vs JSON
                     self.headers['Content-Type'] = 'application/octet-stream'
-                if data and self.headers['Content-Type'] == 'multipart/form-data':
+                if data and len(data) > 0 and self.headers['Content-Type'] == 'multipart/form-data':
                     # convert from params to files (i.e multipart/form-data)
-                    files = {}
+                    if files is None:
+                        files = set()
                     for k,v in data.items():
-                        # files[k] = (None, v, 'application/json')
-                        files[k] = (None, json.dumps(v), 'application/json')
+                        if isinstance(v, (dict, list)):
+                            files.add((k, (None, json.dumps(v), 'application/json')))
+                        else:
+                            files.add((k, (None, v)))
+                    # we have replaced data's values into files
                     data = None
+                if data is not None and len(data) == 0:
+                    data = None
+                if files is not None and len(files) == 0:
+                    files = None
+                if data is None and files is None and self.headers['Content-Type'] == 'multipart/form-data':
+                    # can't have zero length multipart/form-data and as there's no data or files; we don't need it
+                    del self.headers['Content-Type']
                 if files:
                     # overwrite Content-Type as we are uploading data
                     self.headers['Content-Type'] = 'multipart/form-data'
                     # however something isn't right and this works ... look at again later!
                     del self.headers['Content-Type']
+            return data, files
 
         def _add_auth_headers(self, method):
             """ Add authentication headers """
@@ -170,7 +196,7 @@ class CloudFlare():
                 raise CloudFlareAPIError(0, 'no cert token defined')
             self.headers['X-Auth-User-Service-Key'] = api_certtoken
 
-        def do_not_available(self, method, parts, identifiers, params=None, data=None, content_type=None, files=None):
+        def do_not_available(self, method, parts, identifiers, params=None, data=None, files=None, content_type=None):
             """ Cloudflare v4 API"""
 
             # base class simply returns not available - no processing of any arguments
@@ -178,39 +204,39 @@ class CloudFlare():
                 self.logger.debug('call for this method not available')
             raise CloudFlareAPIError(0, 'call for this method not available')
 
-        def do_no_auth(self, method, parts, identifiers, params=None, data=None, content_type=None, files=None):
+        def do_no_auth(self, method, parts, identifiers, params=None, data=None, files=None, content_type=None):
             """ Cloudflare v4 API"""
 
-            self._add_headers(method, content_type, data, files)
+            data, files = self._add_headers(method, data, files, content_type)
             # We decide at this point if we are sending json or string data
             if isinstance(data, (str,bytes,bytearray)):
                 return self._call(method, parts, identifiers, params, data, None, files)
             return self._call(method, parts, identifiers, params, None, data, files)
 
-        def do_auth(self, method, parts, identifiers, params=None, data=None, content_type=None, files=None):
+        def do_auth(self, method, parts, identifiers, params=None, data=None, files=None, content_type=None):
             """ Cloudflare v4 API"""
 
-            self._add_headers(method, content_type, data, files)
+            data, files = self._add_headers(method, data, files, content_type)
             self._add_auth_headers(method)
             # We decide at this point if we are sending json or string data
             if isinstance(data, (str,bytes,bytearray)):
                 return self._call(method, parts, identifiers, params, data, None, files)
             return self._call(method, parts, identifiers, params, None, data, files)
 
-        def do_auth_unwrapped(self, method, parts, identifiers, params=None, data=None, content_type=None, files=None):
+        def do_auth_unwrapped(self, method, parts, identifiers, params=None, data=None, files=None, content_type=None):
             """ Cloudflare v4 API"""
 
-            self._add_headers(method, content_type, data, files)
+            data, files = self._add_headers(method, data, files, content_type)
             self._add_auth_headers(method)
             # We decide at this point if we are sending json or string data
             if isinstance(data, (str,bytes,bytearray)):
                 return self._call_unwrapped(method, parts, identifiers, params, data, None, files)
             return self._call_unwrapped(method, parts, identifiers, params, None, data, files)
 
-        def do_certauth(self, method, parts, identifiers, params=None, data=None, content_type=None, files=None):
+        def do_certauth(self, method, parts, identifiers, params=None, data=None, files=None, content_type=None):
             """ Cloudflare v4 API"""
 
-            self._add_headers(method, content_type, data, files)
+            data, files = self._add_headers(method, data, files, content_type)
             self._add_certtoken_headers(method)
             # We decide at this point if we are sending json or string data
             if isinstance(data, (str,bytes,bytearray)):
@@ -258,21 +284,12 @@ class CloudFlare():
                     if len(parts) > 4 and parts[4]:
                         url += '/' + parts[4]
 
-            if files and data_json:
-                # Can't send data_json and form data - so move data_json into files and send as multipart/form-data
-                new_files = []
-                new_files += [(f, (files[f].name, files[f])) for f in files]
-                new_files += [(d, (None, data_json[d])) for d in data_json]
-                files = tuple(new_files)
-                data_json = None
-
             if self.logger:
                 msg = build_curl(method, url, headers, params, data_str, data_json, files)
                 self.logger.debug('Call: emulated curl command ...\n%s', msg)
 
             try:
                 response = self.network(method, url, headers, params, data_str, data_json, files)
-
             except requests_ConnectionError as e:
                 if self.logger:
                     self.logger.debug('Call: requests connection exception! "%s"', e)
@@ -438,8 +455,8 @@ class CloudFlare():
                     # NDJSON is a series of JSON elements with newlines between each element
                     try:
                         r = []
-                        for l in response_data.splitlines():
-                            r.append(json.loads(l))
+                        for line in response_data.splitlines():
+                            r.append(json.loads(line))
                         response_data = r
                     except (ValueError, json.decoder.JSONDecodeError):
                         # While this should not happen; it's always possible
@@ -569,8 +586,8 @@ class CloudFlare():
                     message = errors['error']
                 else:
                     message = ''
-                ##if 'messages' in response_data:
-                ##    errors['error_chain'] = response_data['messages']
+                # if 'messages' in response_data:
+                #     errors['error_chain'] = response_data['messages']
                 if 'error_chain' in errors:
                     error_chain = errors['error_chain']
                     for error in error_chain:
@@ -643,7 +660,7 @@ class CloudFlare():
                     self.logger.debug('OpenAPI bad json file: %s', e)
                 raise CloudFlareAPIError(0, 'OpenAPI bad json file: %s' % (e)) from None
 
-            #if self.base_url != cloudflare_url:
+            # if self.base_url != cloudflare_url:
             #    # XXX/TODO should this be recorded or throw an error?
             #    pass
 
@@ -674,7 +691,8 @@ class CloudFlare():
 
             self._base = base
             self._parts = parts
-            self._content_type = content_type
+            if content_type:
+                self._content_type = content_type
             self._do = self._base.do_not_available
 
         def __call__(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
@@ -696,27 +714,52 @@ class CloudFlare():
         def get(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
-            return self._do('GET', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data, self._content_type)
+            try:
+                if getattr(self, '_content_type', False):
+                    return self._do('GET', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data, self._content_type)
+                return self._do('GET', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data)
+            except CloudFlareAPIError as e:
+                raise CloudFlareAPIError(e=e) from None
 
         def patch(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
-            return self._do('PATCH', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data, self._content_type)
+            try:
+                if getattr(self, '_content_type', False):
+                    return self._do('PATCH', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data, self._content_type)
+                return self._do('PATCH', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data)
+            except CloudFlareAPIError as e:
+                raise CloudFlareAPIError(e=e) from None
 
         def post(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
-            return self._do('POST', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data, self._content_type, files)
+            try:
+                if getattr(self, '_content_type', False):
+                    return self._do('POST', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data, files, self._content_type)
+                return self._do('POST', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data, files)
+            except CloudFlareAPIError as e:
+                raise CloudFlareAPIError(e=e) from None
 
         def put(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
-            return self._do('PUT', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data, self._content_type, files)
+            try:
+                if getattr(self, '_content_type', False):
+                    return self._do('PUT', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data, files, self._content_type)
+                return self._do('PUT', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data, files)
+            except CloudFlareAPIError as e:
+                raise CloudFlareAPIError(e=e) from None
 
         def delete(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
-            return self._do('DELETE', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data, self._content_type)
+            try:
+                if getattr(self, '_content_type', False):
+                    return self._do('DELETE', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data, self._content_type)
+                return self._do('DELETE', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data)
+            except CloudFlareAPIError as e:
+                raise CloudFlareAPIError(e=e) from None
 
     class _CFbaseUnused(_CFbase):
         """ Cloudflare v4 API"""
@@ -740,22 +783,42 @@ class CloudFlare():
         def patch(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
-            return self._base.do_not_available('PATCH', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data, self._content_type)
+            try:
+                if getattr(self, '_content_type', False):
+                    return self._base.do_not_available('PATCH', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data, self._content_type)
+                return self._base.do_not_available('PATCH', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data)
+            except CloudFlareAPIError as e:
+                raise CloudFlareAPIError(e=e) from None
 
         def post(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
-            return self._base.do_not_available('POST', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data, self._content_type, files)
+            try:
+                if getattr(self, '_content_type', False):
+                    return self._base.do_not_available('POST', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data, files, self._content_type)
+                return self._base.do_not_available('POST', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data, files)
+            except CloudFlareAPIError as e:
+                raise CloudFlareAPIError(e=e) from None
 
         def put(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None, files=None):
             """ Cloudflare v4 API"""
 
-            return self._base.do_not_available('PUT', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data, self._content_type)
+            try:
+                if getattr(self, '_content_type', False):
+                    return self._base.do_not_available('PUT', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data, self._content_type)
+                return self._base.do_not_available('PUT', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data)
+            except CloudFlareAPIError as e:
+                raise CloudFlareAPIError(e=e) from None
 
         def delete(self, identifier1=None, identifier2=None, identifier3=None, identifier4=None, params=None, data=None):
             """ Cloudflare v4 API"""
 
-            return self._base.do_not_available('DELETE', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data, self._content_type)
+            try:
+                if getattr(self, '_content_type', False):
+                    return self._base.do_not_available('DELETE', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data, self._content_type)
+                return self._base.do_not_available('DELETE', self._parts, [identifier1, identifier2, identifier3, identifier4], params, data)
+            except CloudFlareAPIError as e:
+                raise CloudFlareAPIError(e=e) from None
 
     class _CFbaseAuth(_CFbase):
         """ Cloudflare v4 API"""
@@ -787,6 +850,23 @@ class CloudFlare():
             self._do = self._base.do_certauth
             self._valid = True
 
+    @classmethod
+    def sanitize_verb(cls, v):
+        """ sanitize_verb """
+        # keywords are also changed to have underscore appended so it can used with Python code
+        if keyword.iskeyword(v):
+            v = v + '_'
+        # AI functions introduce '@' symbol - i.e .../@cf/... they are replaced with at_
+        if '@' == v[0]:
+            v = 'at_' + v[1:]
+        # AI functions introduce '.' symbol - i.e 1.0 they are replaced with underscore
+        if '.' in v:
+            v = v.replace('.','_')
+        # dashes (vs underscores) cause issues in Python and other languages. they are replaced with underscores
+        if '-' in v:
+            v = v.replace('-','_')
+        return v
+
     def add(self, t, p1, p2=None, p3=None, p4=None, p5=None, content_type=None):
         """ add api call to class"""
 
@@ -808,10 +888,7 @@ class CloudFlare():
         element = None
         for element in a[0:-1]:
             try:
-                if '-' in element:
-                    branch = getattr(branch, element.replace('-','_'))
-                else:
-                    branch = getattr(branch, element)
+                branch = getattr(branch, CloudFlare.sanitize_verb(element))
             except AttributeError:
                 # missing path - should never happen unless api_v4 is a busted file
                 branch = None
@@ -822,15 +899,7 @@ class CloudFlare():
 
         name = a[-1]
         try:
-            if keyword.iskeyword(name):
-                ## add the keyword appended with an extra underscore so it can used with Python code
-                f = getattr(branch, name + '_')
-            else:
-                if '-' in name:
-                    # dashes (vs underscores) cause issues in Python and other languages
-                    f = getattr(branch, name.replace('-','_'))
-                else:
-                    f = getattr(branch, name)
+            branch = getattr(branch, CloudFlare.sanitize_verb(name))
             # we only are here becuase the name already exists - don't let it overwrite - should never happen unless api_v4 is a busted file
             raise CloudFlareAPIError(0, 'api load: duplicate name found: %s/**%s**' % ('/'.join(a[0:-1]), name))
         except AttributeError:
@@ -851,15 +920,19 @@ class CloudFlare():
             # should never happen
             raise CloudFlareAPIError(0, 'api load type mismatch')
 
-        if keyword.iskeyword(name):
-            ## add the keyword appended with an extra underscore so it can used with Python code
-            setattr(branch, name + '_', f)
-        else:
-            if '-' in name:
-                # dashes (vs underscores) cause issues in Python and other languages
-                setattr(branch, name.replace('-','_'), f)
-            else:
-                setattr(branch, name, f)
+        setattr(branch, CloudFlare.sanitize_verb(name), f)
+
+    def find(self, cmd):
+        """ find """
+        m = self
+        for verb in cmd.split('/'):
+            if verb == '' or verb[0] == ':':
+                continue
+            try:
+                m = getattr(m, CloudFlare.sanitize_verb(verb))
+            except AttributeError:
+                raise AttributeError('%s: not found' % (verb)) from None
+        return m
 
     def api_list(self):
         """ recursive walk of the api tree returning a list of api calls"""
@@ -888,7 +961,7 @@ class CloudFlare():
                 if 'delete' in d or 'get' in d or 'patch' in d or 'post' in d or 'put' in d:
                     # only show the result if a call exists for this part
                     if n[-1] == '_':
-                        if  keyword.iskeyword(n[:-1]):
+                        if keyword.iskeyword(n[:-1]):
                             # should always be a keyword - but now nothing needs to be done
                             pass
                         # remove the extra keyword postfix'ed with underscore
