@@ -6,17 +6,7 @@ import sys
 sys.path.insert(0, os.path.abspath('.'))
 import CloudFlare
 
-def find_call(cf, verbs):
-    # So we walk over the @ via a getattr() call.
-    # We also have to deal with a . in a verb - that does not work in Python. So sad.
-    # Also, the - is actually an _ in this Python library. 
-    # This is not normally needed for other calls
-    m = cf
-    for verb in verbs.split('/'):
-        if verb == '' or verb[0] == ':':
-            continue
-        m = getattr(m, verb)
-    return m
+use_find = False
 
 def doit(account_name, english_text):
 
@@ -27,7 +17,7 @@ def doit(account_name, english_text):
     cf = CloudFlare.CloudFlare(global_request_timeout=120)
 
     try:
-        if account_name == None or account_name == '':
+        if account_name is None or account_name == '':
             params = {'per_page': 1}
         else:
             params = {'name': account_name, 'per_page': 1}
@@ -47,10 +37,13 @@ def doit(account_name, english_text):
     }
 
     try:
-        # This should be easy to call; however, the @ will not work in Python (or many languages)
-        # r = cf.accounts.ai.run.@cf.meta.m2m100-1.2b(account_id, data=translate_data)
-        # We find the endpoint via a quick string search
-        r = find_call(cf, '/accounts/:id/ai/run/@cf/meta/m2m100_1.2b').post(account_id, data=translate_data)
+        if use_find:
+            # you can use this format:
+            r = cf.find('/accounts/:id/ai/run/@cf/meta/m2m100-1.2b').post(account_id, data=translate_data)
+        else:
+            # or you can use this format:
+            # @'s are replaced by at_ so .../@cf/... becomes .../at_cf/... etc
+            r = cf.accounts.ai.run.at_cf.meta.m2m100_1_2b.post(account_id, data=translate_data)
     except CloudFlare.exceptions.CloudFlareAPIError as e:
         exit('/ai.run %d %s - api call failed' % (e, e))
 
